import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {
  AbstractControl,
  FormArray,
  FormControl,
  FormGroup,
  FormsModule,
  ReactiveFormsModule, ValidationErrors,
  Validators
} from "@angular/forms";
import {Hotel} from "../model/hotel";
import {RouterLink} from "@angular/router";
import {NgForOf, NgIf} from "@angular/common";
import {debounceTime} from "rxjs";
import {ErrorMsgComponent} from "./error-msg.component";

@Component({
  selector: 'app-edit-hotel',
  standalone: true,
  imports: [
    FormsModule,
    ReactiveFormsModule,
    RouterLink,
    NgForOf,
    ErrorMsgComponent,
    NgIf,
  ],
  template: `
    <ng-form [formGroup]="form" (ngSubmit)="submit()">
      <label for="name">Name</label>
      <br>
      <input id="name" type="text" formControlName="name">
      <app-error-msg [msg]="errorMsgs['name']"></app-error-msg>
      <br>
      <label for="description">Description</label>
      <br>
      <input id="description" type="text" formControlName="description">
      <app-error-msg [msg]="errorMsgs['description']"></app-error-msg>
      <br>
      <label for="price">Price</label>
      <br>
      <input id="price" type="number" formControlName="price">
      <app-error-msg [msg]="errorMsgs['price']"></app-error-msg>
      <br>
      <label for="rating">Rating</label>
      <br>
      <input id="rating" type="number" formControlName="rating">
      <app-error-msg [msg]="errorMsgs['rating']"></app-error-msg>
      <br>
      <br>
      <label for="email">Email</label>
      <input id="email" type="checkbox" name="contactType" value="Email" [checked]="activeContact == 'email'"
             (click)="changeContactType('email')">
      <br>
      <label for="phoneNumber">Phone Number</label>
      <input id="phoneNumber" type="checkbox" name="contactType" value="PhoneNumber"
             [checked]="activeContact == 'phoneNumber'" (click)="changeContactType('phoneNumber')">
      <br>
      <label for="none">None</label>
      <input id="none" type="checkbox" name="contactType" value="none" [checked]="activeContact == 'none'"
             (click)="changeContactType('none')">

      <br>
      <br>

      @if (activeContact === 'email') {
        <app-error-msg [msg]="errorMsgs['emailGroup']"></app-error-msg>
        <div formGroupName="emailGroup">
          <app-error-msg [msg]="errorMsgs['email']"></app-error-msg>
          <br>
          <label for="email">Email</label>
          <br>
          <input type="email" formControlName="email" required>
          <br>
          <label for="emailConfirm">Confirm Email</label>
          <app-error-msg [msg]="errorMsgs['emailConfirm']"></app-error-msg>
          <br>
          <input type="email" formControlName="emailConfirm" required>
        </div>
      } @else if (activeContact === 'phoneNumber') {
        <app-error-msg [msg]="errorMsgs['phoneNumberGroup']"></app-error-msg>
        <div formGroupName="phoneNumberGroup">
          <app-error-msg [msg]="errorMsgs['phoneNumber']"></app-error-msg>
          <br>
          <label for="phoneNumber">Phone Number</label>
          <br>
          <input type="number" formControlName="phoneNumber" required>
          <br>
          <label for="phoneNumberConfirm">Confirm Phone Number</label>
          <app-error-msg [msg]="errorMsgs['phoneNumberConfirm']"></app-error-msg>
          <br>
          <input type="number" formControlName="phoneNumberConfirm" required>
        </div>
      }

      <br>
      <br>
      <button (click)="addTag()">+</button>
      <div formArrayName="tags">
        <div *ngFor="let tag of getTags().controls; let i = index">
          <input type="text" placeholder="tag" [formControlName]="i">
          <button (click)="deleteTag(tag)">-</button>
        </div>
        <app-error-msg [msg]="errorMsgs['tags']"></app-error-msg>
      </div>
      <br>
      <br>
      <button type="submit" (click)="submit()">Submit</button>
    </ng-form>

    <button routerLink="/hotels">back</button>
  `,
})
export class EditHotelComponent implements OnInit {
  errorMsgs: Record<string, string> = {};

  @Input()
  hotel: Hotel | null = null;

  @Output()
  updateHotel: EventEmitter<Hotel> = new EventEmitter

  form!: FormGroup

  validationErrors: Record<string, string> = {
    required: 'This field is required',
    min: 'Value must be greater than or equal to 0',
    max: 'Value must be less than or equal to 5',
    emailMatch: 'Emails do not match',
    phoneNumberMatch: 'Phone numbers do not match',
  };

  activeContact!: string;

  ngOnInit(): void {
    if (this.hotel?.email) {
      this.activeContact = 'email';
    } else if (this.hotel?.phoneNumber) {
      this.activeContact = 'phoneNumber';
    } else {
      this.activeContact = 'none';
    }

    const tags = [];

    for (const tag of this.hotel?.tags ?? []) {
      tags.push(new FormControl(tag, [Validators.required]));
    }

    this.form = new FormGroup({
      name: new FormControl(this.hotel?.hotelName, [Validators.required]),
      description: new FormControl(this.hotel?.description, [Validators.required]),
      price: new FormControl(this.hotel?.price, [Validators.required]),
      rating: new FormControl(this.hotel?.rating, [Validators.required, Validators.min(0), Validators.max(5)]),
      emailGroup: new FormGroup({
        email: new FormControl(this.hotel?.email, [Validators.required, Validators.email]),
        emailConfirm: new FormControl(this.hotel?.email, [Validators.required, Validators.email]),
      }, {validators: this.emailMatch}),
      phoneNumberGroup: new FormGroup({
        phoneNumber: new FormControl(this.hotel?.phoneNumber, [Validators.required, Validators.maxLength(10), Validators.minLength(10)]),
        phoneNumberConfirm: new FormControl(this.hotel?.phoneNumber, [Validators.required, Validators.maxLength(10), Validators.minLength(10)]),
      }, {validators: this.phoneNumberMatch}),
      tags: new FormArray(tags),
    })

    Object.keys(this.form.controls).forEach(controlName => {
      let debounce = 1000;
      const control = this.form.get(controlName);
      if (!control) {
        return;
      }

      if (controlName === 'rating') {
        debounce = 0;
      }

      control?.valueChanges?.pipe(debounceTime(debounce)).subscribe(() => {
        console.log(controlName, control)
        this.setErrorMessage(controlName, control)
      });
    });
  }

  submit() {
    if (!this.form.valid) {
      this.showErrors();
      return;
    }

    const hotel: Hotel = {
      imageUrl: this.hotel?.imageUrl ?? "",
      hotelName: this.form.value.name,
      description: this.form.value.description,
      price: this.form.value.price,
      rating: this.form.value.rating,
      id: this.hotel?.id ?? 0,
      tags: this.form.value.tags ?? [],
      email: this.form.value.email,
      phoneNumber: this.form.value.phoneNumber
    };

    this.updateHotel.emit(hotel)

    this.hotel = hotel;
  }

  getTags() {
    return this.form.controls['tags'] as FormArray;
  }

  addTag() {
    this.getTags().push(new FormControl('', [Validators.required]));
  }

  deleteTag(tagElement: AbstractControl) {
    this.getTags().removeAt(this.getTags().controls.indexOf(tagElement));
  }

  setErrorMessage(controlName: string, control: AbstractControl) {
    this.errorMsgs[controlName] = Object.keys(control.errors ?? {}).map ((key) => this.validationErrors[key]).join(' ');
    console.log(this.errorMsgs[controlName])
  }

  showErrors() {
    Object.keys(this.form.controls).forEach(controlName => {
      const control = this.form.get(controlName);
      if (!control) {
        return;
      }

      this.setErrorMessage(controlName, control);
    });
  }

  changeContactType(type: string) {
    this.activeContact = type;
  }

  emailMatch(control: AbstractControl): ValidationErrors | null {
    const email = control.get('email');
    const emailConfirm = control.get('emailConfirm');

    if (!emailConfirm || !email) {
      return null;
    }

    if (email.value !== emailConfirm.value) {
      return {emailMatch: true};
    }

    return null;
  }

  phoneNumberMatch(control: AbstractControl) {
    const phoneNumber = control.get('phoneNumber');
    const phoneNumberConfirm = control.get('phoneNumberConfirm');

    if (!phoneNumberConfirm || !phoneNumber) {
      return null;
    }

    if (phoneNumber.value !== phoneNumberConfirm.value) {
      return {phoneNumberMatch: true};
    }

    return null;
  }
}
